@@ -48,6 +48,11 @@ my $ARCH_REGEX = Lintian::Data->new('binaries/arch-regex', qr/\s*\~\~/o,
     sub { return qr/$_[1]/ });
 my $ARCH_64BIT_EQUIVS
   = Lintian::Data->new('binaries/arch-64bit-equivs', qr/\s*\=\>\s*/);
+my $BINARY_SPELLING_EXCEPTIONS
+  = Lintian::Data->new('binaries/spelling-exceptions', qr/\s+/);
+
+my %PATH_DIRECTORIES = map { $_ => 1 } qw(
+  bin/ sbin/ usr/bin/ usr/sbin/ usr/games/ );
 
 sub _embedded_libs {
     my ($key, $val, undef) = @_;
@@ -187,7 +192,8 @@ sub run {
             # According to the binutils documentation[1], the profiling symbol
             # can be named "mcount", "_mcount" or even "__mcount".
             # [1] http://sourceware.org/binutils/docs/gprof/Implementation.html
-            if ($sec =~ /^GLIBC_.*/ and $sym =~ m{\A _?+ _?+ mcount \Z}xsm){
+            if (    $sec =~ /^GLIBC_.*/
+                and $sym =~ m{\A _?+ _?+ (gnu_)?+mcount(_nc)?+ \Z}xsm) {
                 $is_profiled = 1;
             } elsif ($arch ne 'hppa') {
                 # This code was used to detect profiled code in Wheezy
@@ -310,7 +316,7 @@ sub run {
         if ($multiarch eq 'same') {
             unless ($fname
                 =~ m,\b$gnu_triplet_re(?:\b|_)|/(?:$ruby_triplet_re|java-\d+-openjdk-\Q$arch\E|\.build-id)/,
-              ) {
+            ) {
                 tag 'arch-dependent-file-not-in-arch-specific-directory',$file;
             }
         }
@@ -342,6 +348,11 @@ sub run {
         # ELF?
         next unless $fileinfo =~ m/^[^,]*\bELF\b/o;
 
+        tag 'development-package-ships-elf-binary-in-path', $file
+          if exists($PATH_DIRECTORIES{$file->dirname})
+          and $info->field('section', 'NONE') =~ m/(?:^|\/)libdevel$/
+          and $info->field('multi-arch', 'NONE') ne 'foreign';
+
         $objdump = $info->objdump_info->{$fname};
 
         if ($arch eq 'all' or not $ARCH_REGEX->known($arch)) {
@@ -378,13 +389,7 @@ sub run {
         my $strings = slurp_entire_file($info->strings($file));
         my $exceptions = {
             %{ $group->info->spelling_exceptions },
-            'teH' => 1, # From #711207
-            'tEH' => 1, # From #782902
-            'tEh' => 1, # From #782902, too
-            'ang' => 1, # The Go stdlib html/ package contains "ang;"
-            'writeN' => 1, # The Go stdlib text/tabwriter pkg contains "writeN"
-            'writeN' => 1, # The Go stdlib text/tabwriter pkg contains "writeN"
-            'ot' => 1, # The Go stdlib runtime/ package contains "ot"
+            map { $_ => 1} $BINARY_SPELLING_EXCEPTIONS->all
         };
         my $tag_emitter
           = spelling_tag_emitter('spelling-error-in-binary', $file);
@@ -478,7 +483,7 @@ sub run {
             $fname =~ m,usr/lib/(?:pyshared/)?python2\.\d+/.*(?<!_d)\.so$,
             or(     $fname =~ m,usr/lib/python3/.+\.cpython-\d+([a-z]+)\.so$,
                 and $1 !~ /d/)
-          ) {
+        ) {
             if (index($strings, 'numpy') > -1 and $strings =~ NUMPY_REGEX) {
                 $uses_numpy_c_abi = 1;
             }
@@ -509,7 +514,7 @@ sub run {
 
         # statically linked?
         if (!exists($objdump->{NEEDED})) {
-            if ($fileinfo =~ m/shared object/o) {
+            if ($fileinfo =~ m/(shared object|pie executable)/o) {
                 # Some exceptions: kernel modules, syslinux modules, detached
                 # debugging information and the dynamic loader (which itself
                 # has no dependencies).
@@ -517,6 +522,7 @@ sub run {
                 next if ($fname =~ m%^lib/modules/%);
                 next if ($fname =~ m%^usr/lib/debug/%);
                 next if ($fname =~ m%\.(?:[ce]32|e64)$%);
+                next if ($fname =~ m%^usr/lib/jvm/.*\.debuginfo$%);
                 next
                   if (
                     $fname =~ m{
@@ -551,7 +557,7 @@ sub run {
             my $is_shared = 0;
             my @needed;
             $needs_depends_line = 1;
-            $is_shared = 1 if index($fileinfo, 'shared object') != -1;
+            $is_shared = 1 if $fileinfo =~ m/(shared object|pie executable)/;
             @needed = @{$objdump->{NEEDED}} if exists($objdump->{NEEDED});
             for my $lib (@needed) {
                 if ($lib =~ /^libc\.so\.(\d+.*)/) {
