@@ -5,7 +5,7 @@
 # Copyright (C) 2000 Sean 'Shaleh' Perry
 # Copyright (C) 2002 Josip Rodin
 # Copyright (C) 2007 Russ Allbery
-# Copyright (C) 2013-2014 Bastien ROUCARIÈS
+# Copyright (C) 2013-2018 Bastien ROUCARIÈS
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,10 +39,13 @@ use constant BLOCKSIZE => 16_384;
 use constant INSANE_LINE_LENGTH => 512;
 use constant SAFE_LINE_LENGTH => 256;
 
+use File::Basename qw(basename);
+use List::MoreUtils qw(any);
+
 use Lintian::Data;
 use Lintian::Relation ();
 use Lintian::Tags qw(tag);
-use Lintian::Util qw(internal_error normalize_pkg_path strip);
+use Lintian::Util qw(internal_error normalize_pkg_path strip open_gz);
 use Lintian::SlidingWindow;
 
 our $LIBTOOL = Lintian::Relation->new('libtool | dh-autoreconf');
@@ -151,6 +154,9 @@ my $RFC_WHITELIST =  Lintian::Data->new(
         return qr/$_[0]/xms;
     });
 
+# "Known good" files that match eg. lena.jpg.
+my $LENNA_WHITELIST = Lintian::Data->new('cruft/lenna-whitelist');
+
 # prebuilt-file or forbidden copyright
 my $BAD_LINK_COPYRIGHT =  Lintian::Data->new(
     'cruft/bad-link-copyright',
@@ -171,6 +177,14 @@ sub _minified_javascript_name_regexp {
       : qr/(?i)[-._](?:min|pack(?:ed)?)\.js$/;
 }
 
+# get browserified regexp
+my $BROWSERIFY_REGEX =  Lintian::Data->new(
+    'cruft/browserify-regex',
+    qr/\s*\~\~\s*/,
+    sub {
+        return qr/$_[1]/xms;
+    });
+
 sub _get_license_check_file {
     my ($filename) = @_;
     my $data = Lintian::Data->new(
@@ -324,8 +338,15 @@ my @file_checks = (
 # to the debian/ source directory
 our @EOL_TERMINATORS_FILES = qw(control changelog);
 
+# List of files to check for a trailing whitespace characters
+our @TRAILING_WHITESPACE_FILES = (
+    ['debian/changelog'        => qr,\s+\n$,],
+    ['debian/control'          => qr,\s+\n$,],
+    ['debian/rules'            => qr,[ ]+\n$,], # Allow trailing tabs in Make
+);
+
 sub run {
-    my (undef, undef, $info, $proc) = @_;
+    my (undef, undef, $info, $proc, $group) = @_;
     my $source_pkg = $proc->pkg_src;
     my $d_files = $info->index_resolved_path('debian/files');
 
@@ -333,19 +354,8 @@ sub run {
         tag 'debian-files-list-in-source';
     }
 
-    # This doesn't really belong here, but there isn't a better place at the
-    # moment to put this check.
-    my $version = $info->field('version', '0-1');
-
-    if ($info->native) {
-        if ($version =~ /-/ and $version !~ /-0\.[^-]+$/) {
-            tag 'native-package-with-dash-version';
-        }
-    }else {
-        if ($version !~ /-/) {
-            tag 'non-native-package-with-native-version';
-        }
-    }
+    tag 'package-uses-deprecated-source-override-location'
+      if $info->index_resolved_path('debian/source.lintian-overrides');
 
     # Check if the package build-depends on autotools-dev, automake,
     # or libtool.
@@ -362,7 +372,7 @@ sub run {
     }elsif (not $info->native) {
         check_diffstat($info->diffstat, \%warned);
     }
-    find_cruft($source_pkg, $info, \%warned, $ltinbd);
+    find_cruft($source_pkg, $info, \%warned, $ltinbd, $group);
 
     for my $file (@EOL_TERMINATORS_FILES) {
         my $path = $info->index_resolved_path("debian/$file");
@@ -377,10 +387,34 @@ sub run {
         close($fd);
     }
 
+    for my $file (@TRAILING_WHITESPACE_FILES) {
+        my $path = $info->index_resolved_path($file->[0]);
+        next if not $path or not $path->is_open_ok;
+        my $fd = $path->open;
+        my @empty_lines;
+        while (my $line = <$fd>) {
+            if ($line eq "\n") {
+                push @empty_lines, $.;
+                next;
+            }
+            @empty_lines = (); # reset; line is not empty
+            tag 'file-contains-trailing-whitespace', "$path (line $.)"
+              if ($line =~ $file->[1]);
+        }
+        close($fd);
+        for my $num (@empty_lines) {
+            tag 'file-contains-trailing-whitespace', "$path (line $num)";
+        }
+    }
+
     if (my $pycompat = $info->index_resolved_path('debian/pycompat')) {
         tag 'debian-pycompat-is-obsolete' if $pycompat->is_file;
     }
 
+    if (my $pyversions = $info->index_resolved_path('debian/pyversions')) {
+        tag 'debian-pyversions-is-obsolete' if $pyversions->is_file;
+    }
+
     # Report any error messages from tar while unpacking the source
     # package if it isn't just tar cruft.
     for my $file (keys %ERRORS) {
@@ -411,6 +445,12 @@ sub run {
         }
     }
 
+    foreach my $file ($info->sorted_orig_index) {
+        tag 'source-contains-empty-directory', $file
+          if $file->is_dir
+          and scalar($file->children) == 0;
+    }
+
     return;
 }    # </run>
 
@@ -543,9 +583,10 @@ sub istestset {
 # "source-contains" tag.  The tag isn't entirely accurate, but it's better
 # than creating yet a third set of tags, and this gets the severity right.
 sub find_cruft {
-    my ($source_pkg, $info, $warned, $ltinbd) = @_;
+    my ($source_pkg, $info, $warned, $ltinbd, $group) = @_;
     my $prefix = ($info->native ? 'diff-contains' : 'source-contains');
     my @worklist;
+    my $ships_examples = _ships_examples($group);
 
     # start with the top-level dirs
     push(@worklist, $info->index('')->children);
@@ -583,6 +624,8 @@ sub find_cruft {
                         last;
                     }
                 }
+                tag 'package-does-not-install-examples', $entry
+                  if $basename eq 'examples' and not $ships_examples;
             }
 
             push(@worklist, $entry->children);
@@ -613,7 +656,7 @@ sub find_cruft {
                 $name, $md5sum, $NON_DISTRIBUTABLE_FILES,
                 'license-problem-md5sum-non-distributable-file'
             )
-          ) {
+        ) {
             next ENTRY;
         }
         unless ($info->is_non_free) {
@@ -655,8 +698,26 @@ sub find_cruft {
         # Ensure we have a README.source for R data files
         if (   $basename =~ m,\.(?:rda|Rda|rdata|Rdata|RData)$,
             && $entry->is_file
+            && $entry->is_open_ok
+            && $file_info =~ /gzip compressed data/
             && !$info->index_resolved_path('debian/README.source')) {
-            tag 'r-data-without-readme-source', $name;
+            my $fd = $entry->open_gz;
+            read($fd, my $magic, 4);
+            close($fd);
+            tag 'r-data-without-readme-source', $name if $magic eq 'RDX2';
+        }
+
+        if (   $name =~ m,configure.(in|ac)$,
+            && $entry->is_file
+            && $entry->is_open_ok) {
+            my $fd = $entry->open;
+            while (my $line = <$fd>) {
+                next if $line =~ m{^\s*dnl};
+                tag 'autotools-pkg-config-macro-not-cross-compilation-safe',
+                  $name, "(line $.)"
+                  if $line=~ m{AC_PATH_PROG\s*\([^,]+,\s*\[?pkg-config\]?\s*,};
+            }
+            close($fd);
         }
 
         # Lena Söderberg image
@@ -666,7 +727,8 @@ sub find_cruft {
                 or $file_info =~ /\bbitmap\b/i
                 or $file_info =~ /^PDF Document\b/i
                 or $file_info =~ /^Postscript Document\b/i) {
-                tag 'license-problem-non-free-img-lenna', $name;
+                tag 'license-problem-non-free-img-lenna', $name
+                  unless $LENNA_WHITELIST->known($md5sum);
             }
 
         }
@@ -703,7 +765,7 @@ sub find_cruft {
             if (
                 index($contents,
                     'You WILL either need to modify or delete this file') >= 0
-              ) {
+            ) {
                 tag 'readme-source-is-dh_make-template';
             }
         }
@@ -726,6 +788,7 @@ sub find_cruft {
             # debian/changelog
             if (   $short ne 'changelog'
                 && $name !~ m{^debian/patches/.*$}
+                && $name !~ m{^debian/(?:.+\.)?install$}
                 && $entry->is_file
                 && $entry->is_open_ok) {
                 my $contents = $entry->file_contents;
@@ -738,12 +801,9 @@ sub find_cruft {
             }
         }
 
-        if (   $basename eq 'doxygen.png'
-            or $basename eq 'doxygen.sty') {
-            unless ($source_pkg eq 'doxygen') {
-                tag 'source-contains-prebuilt-doxygen-documentation', $dirname;
-            }
-        }
+        tag 'source-contains-prebuilt-doxygen-documentation', $dirname
+          if $basename =~ m{^doxygen.(?:png|sty)$}
+          and $source_pkg ne 'doxygen';
 
         unless ($warned->{$name}) {
             for my $rule (@file_checks) {
@@ -787,6 +847,7 @@ sub check_missing_source {
     my ($file, $info, $name, $basename, $dirname,$replacementspairref,
         $extratext)
       = @_;
+    my $basename_of_dirname = basename($dirname);
     $extratext //= '';
 
     # do not check missing source for non free
@@ -825,6 +886,7 @@ sub check_missing_source {
             my $newpath;
             # first replace dir name
             $path =~ s/\$dirname/$dirname/g;
+            $path =~ s/\$basename_of_dirname/$basename_of_dirname/g;
             # absolute path
             if(substr($path,0,1) eq '/') {
                 $path =~ s,^/+,,g;
@@ -847,8 +909,8 @@ sub check_missing_source {
             if($newpath eq '') {
                 next PATH;
             }
-            # found source return
-            if($info->index($newpath)) {
+            # found source file or directory
+            if($info->index_resolved_path($newpath)) {
                 return;
             }
         }
@@ -880,6 +942,9 @@ sub full_text_check {
         return;
     }
 
+    my $ishtml = ($basename =~ m,\.(?:x?html?\d?|xht)$,i);
+    my $skiphtml = 0;
+
     # some js file comments are really really long
     my $sfd= Lintian::SlidingWindow->new($fd, \&lc_block, BLOCKSIZE);
     my %licenseproblemhash;
@@ -898,14 +963,14 @@ sub full_text_check {
         # permission to distribute those.
         if(
             _license_check(
-                $source_pkg, $name,
-                $basename,$NON_DISTRIBUTABLE_LICENSES,
-                $block,$blocknumber,
-                \$cleanedblock,\%matchedkeyword,
-                \%licenseproblemhash
+                $info, $source_pkg,
+                $name,$basename,
+                $NON_DISTRIBUTABLE_LICENSES,$block,
+                $blocknumber,\$cleanedblock,
+                \%matchedkeyword,\%licenseproblemhash
             )
-          ){
-            return;
+        ){
+            last BLOCK;
         }
 
         # Skip the rest of the license checks for non-free
@@ -915,11 +980,23 @@ sub full_text_check {
         }
 
         _license_check(
-            $source_pkg, $name, $basename,
-            $NON_FREE_LICENSES,$block,  $blocknumber,
-            \$cleanedblock, \%matchedkeyword,\%licenseproblemhash
+            $info, $source_pkg, $name,
+            $basename,$NON_FREE_LICENSES,$block,
+            $blocknumber,\$cleanedblock, \%matchedkeyword,
+            \%licenseproblemhash
         );
 
+        # check html
+        if($ishtml && !$skiphtml) {
+            if(
+                _check_html_cruft(
+                    $entry, $info, $source_pkg,$name,
+                    $basename,$dirname,$block,$blocknumber
+                ) < 0
+            ) {
+                $skiphtml = 1;
+            }
+        }
         # check only in block 0
         if($blocknumber == 0) {
             _search_in_block0($entry, $info, $name, $basename, $dirname,
@@ -930,6 +1007,75 @@ sub full_text_check {
     return;
 }
 
+# check javascript in html file
+sub _check_html_cruft {
+    my ($entry, $info, $source_pkg,$name,$basename,$dirname,$block,
+        $blocknumber)=@_;
+
+    my $blockscript = $block;
+    my $indexscript;
+
+    if($blocknumber == 0) {
+        if(index($block,'<meta name="generator"') > -1) {
+            if($block =~ m,<meta \s+ name="generator" \s+ content="doxygen,smx)
+            {
+                tag 'source-contains-prebuilt-doxygen-documentation', $entry;
+                return -1;
+            }
+        }
+    }
+
+    while(($indexscript = index($blockscript, '<script')) > -1) {
+        $blockscript = substr($blockscript,$indexscript);
+        # sourced script ok
+        if($blockscript =~  m,\A<script\s+[^>]*?src="[^"]+?"[^>]*?>,sm) {
+            $blockscript = substr($blockscript,$+[0]);
+            next;
+        }
+        # extract script
+        if($blockscript =~ m,<script[^>]*?>(.*?)</script>,sm) {
+            $blockscript = substr($blockscript,$+[0]);
+            if(_check_js_script($entry,$info,$name,$basename,$dirname,$1)) {
+                return 0;
+            }
+            next;
+        }
+        # here we know that we have partial script. Do the check nevertheless
+        # first check if we have the full <script> tag and do the check
+        # if we get <script src="  "
+        # then skip
+        if($blockscript =~ m,\A<script[^>]*?>,sm) {
+            $blockscript = substr($blockscript,$+[0]);
+            _check_js_script($entry,$info,$name,$basename,$dirname,
+                $blockscript);
+        }
+        return 0;
+    }
+    return 1;
+}
+
+# check if js script is minified
+sub _check_js_script {
+    my ($entry, $info, $name,$basename,$dirname,$lcscript) = @_;
+    my $firstline = '';
+    foreach (split /\n/, $lcscript) {
+        if ($_ =~ m/^\s*$/) {
+            next;
+        }else {
+            $firstline=$_;
+            last;
+        }
+    }
+    if ($firstline =~ m/.{0,20}((?:\bcopyright\b|[\(]c[\)]\s*\w|©).{0,50})/) {
+        my $extract = $1;
+        $extract =~ s/^\s+|\s+$//g;
+        tag 'embedded-script-includes-copyright-statement', $name,
+          'extract of copyright statement:',
+          $extract;
+    }
+    return _linelength_test($entry,$info,$name,$basename,$dirname,$lcscript);
+}
+
 # check if file is javascript but not minified
 sub _is_javascript_but_not_minified {
     my ($name) = @_;
@@ -979,6 +1125,11 @@ sub _search_in_block0 {
         elsif ($block =~ m/css_browser_selector\(/) {
             return;
         }
+        # Avoid false-positives in Jush's syntax highlighting definition files.
+        elsif ($block =~ m/jush\.tr\./) {
+            return;
+        }
+
         # now search hidden minified
         _linelength_test($entry, $info, $name, $basename, $dirname,$block);
     }
@@ -1010,9 +1161,15 @@ sub _warn_prebuilt_javascript{
     tag 'source-contains-prebuilt-javascript-object',$name,$extratext;
     # Check for missing source.  It will check
     # for the source file in well known directories
-    check_missing_source($entry,$info,$name,$basename,$dirname,
-        [['(?i)\.js$','.debug.js'],['(?i)\.js$','-debug.js'],['','']],
-        $extratext);
+    if($basename =~ m,\.js$,i) {
+        check_missing_source($entry,$info,$name,$basename,$dirname,
+            [['(?i)\.js$','.debug.js'],['(?i)\.js$','-debug.js'],['','']],
+            $extratext);
+    } else  {
+        # html file
+        check_missing_source($entry,$info,$name,$basename,$dirname,
+            [['$','.fragment.js']],$extratext);
+    }
     return;
 }
 
@@ -1052,6 +1209,21 @@ sub _strip_c_comments {
     return $block;
 }
 
+# detect browserified javascript (comment are removed here and code is stripped)
+sub _detect_browserify {
+    my ($entry, $info, $name, $basename, $dirname, $block) = @_;
+    $block =~ s,\n, ,msg;
+    foreach my $browserifyregex ($BROWSERIFY_REGEX->all) {
+        my $regex = $BROWSERIFY_REGEX->value($browserifyregex);
+        if($block =~ m{$regex}) {
+            my $extra = (defined $1) ? 'code fragment:'.$1 : '';
+            tag 'source-contains-browserified-javascript', $name, $extra;
+            last;
+        }
+    }
+    return;
+}
+
 # try to detect non human source based on line length
 sub _linelength_test {
     my ($entry, $info, $name, $basename, $dirname, $block) = @_;
@@ -1084,13 +1256,16 @@ sub _linelength_test {
         # remove last \n
         $block =~ s/\n\Z//m;
 
+        # detect browserification
+        _detect_browserify($entry, $info, $name, $basename, $dirname, $block);
+
         # retry insane line length test now: if insane length likely minified
         ($linelength)= _linelength_test_maxlength($block,INSANE_LINE_LENGTH);
 
         if($linelength) {
             _warn_prebuilt_javascript($entry, $info, $name, $basename,
                 $dirname,$linelength,INSANE_LINE_LENGTH);
-            return;
+            return 1;
         }
     }
     # Now try to be more clever and work only on the 8192 character
@@ -1104,22 +1279,26 @@ sub _linelength_test {
     # remove last \n
     $strip =~ s/\n\Z//m;
     $nextblock = $strip;
+
+    # detect browserified
+    _detect_browserify($entry, $info, $name, $basename, $dirname, $nextblock);
+
     while(length($nextblock)) {
         # check line above > SAFE_LINE_LENGTH
         ($linelength,$line,$nextblock)
           = _linelength_test_maxlength($nextblock,SAFE_LINE_LENGTH);
         # no long line
         unless($linelength) {
-            return;
+            return 0;
         }
         # compute number of ;
         if(($line =~ tr/;/;/) > 1) {
             _warn_prebuilt_javascript($entry, $info, $name, $basename,
                 $dirname,$linelength,SAFE_LINE_LENGTH);
-            return;
+            return 1;
         }
     }
-    return;
+    return 0;
 }
 
 sub _tag_gfdl {
@@ -1131,10 +1310,10 @@ sub _tag_gfdl {
 # return True in case of license problem
 sub _check_gfdl_license_problem {
     my (
-        $source_pkg, $name,$basename,
-        $block,$blocknumber,$cleanedblock,
-        $matchedkeyword,$licenseproblemhash,$licenseproblem,
-        %matchedhash
+        $info, $source_pkg, $name,
+        $basename,$block,$blocknumber,
+        $cleanedblock,$matchedkeyword,$licenseproblemhash,
+        $licenseproblem,%matchedhash
     )= @_;
     my $rawgfdlsections  = $matchedhash{rawgfdlsections}  || '';
     my $rawcontextbefore = $matchedhash{rawcontextbefore} || '';
@@ -1178,7 +1357,7 @@ sub _check_gfdl_license_problem {
     unless(
         defined(
             $licenseproblemhash->{'license-problem-gfdl-invariants-empty'})
-      ) {
+    ) {
         if ($gfdlsections eq '') {
             # lie in order to check more part
             tag 'license-problem-gfdl-invariants-empty', $name;
@@ -1194,7 +1373,7 @@ sub _check_gfdl_license_problem {
                           [ ]? no [ ] front(?:[ ]?-[ ]?|[ ])cover [ ] texts[ ]?,?
                           [ ]? and [ ] no [ ] back(?:[ ]?-?[ ]?|[ ])cover [ ] texts
                           \Z/xso
-      ) {
+    ) {
         return 0;
     }
 
@@ -1206,7 +1385,7 @@ sub _check_gfdl_license_problem {
                              (?:[ ] using [ ] all [ ] the [ ] features? [ ] of [ ] the [ ] gfdl)?
                             )?
                            )? [ ]? [,:]? \Z/xso
-      ){
+    ){
         return 0;
     }
 
@@ -1244,10 +1423,10 @@ sub _check_gfdl_license_problem {
 # whitelist good rfc
 sub _rfc_whitelist_filename {
     my (
-        $source_pkg, $name, $basename,
-        $block,$blocknumber, $cleanedblock,
-        $matchedkeyword,$licenseproblemhash, $licenseproblem,
-        %matchedhash
+        $info, $source_pkg, $name,
+        $basename,$block,$blocknumber,
+        $cleanedblock,$matchedkeyword,$licenseproblemhash,
+        $licenseproblem,%matchedhash
     )= @_;
     return 0 if $name eq 'debian/copyright';
     my $lcname = lc($basename);
@@ -1265,12 +1444,19 @@ sub _rfc_whitelist_filename {
 # whitelist php source
 sub _php_source_whitelist {
     my (
-        $source_pkg, $name, $basename,
-        $block,$blocknumber, $cleanedblock,
-        $matchedkeyword,$licenseproblemhash, $licenseproblem,
-        %matchedhash
+        $info, $source_pkg, $name,
+        $basename,$block,$blocknumber,
+        $cleanedblock,$matchedkeyword,$licenseproblemhash,
+        $licenseproblem,%matchedhash
     )= @_;
 
+    my $copyright_path = $info->index_resolved_path('debian/copyright');
+    if (    $copyright_path
+        and $copyright_path->file_contents
+        =~ m{^Source: https?://pecl.php.net/package/.*$}m) {
+        return 0;
+    }
+
     if($source_pkg =~ m,^php\d*(?:\.\d+)?$,xms) {
         return 0;
     }
@@ -1303,6 +1489,9 @@ sub _clean_block {
     # po comment may include html tag
     $text =~ s/\"\s?\v\#~\s?\"//gxms;
 
+    # strip .rtf paragraph marks (#892967)
+    $text =~ s/\\par\b//gxms;
+
     $text =~ s/\\url[{][^}]*?[}]/ /gxms;      # (la)?tex url
     $text =~ s/\\emph[{]/ /gxms;              # (la)?tex emph
     $text =~ s/\\href[{][^}]*?[}]
@@ -1338,10 +1527,11 @@ sub _clean_block {
 
     $text =~ s{</?a[^>]*?>}{ }gxms;           # a link
     $text =~ s{<br\s*/?>}{ }gxms;             # (X)?HTML line
-    # breaks
+     # breaks
     $text =~ s{</?citetitle[^>]*?>}{ }gxms;   # DocBook citation title
     $text =~ s{</?div[^>]*?>}{ }gxms;         # html style
-    $text =~ s{</?font[^>]*?>}{ }gxms;        # font
+    $text =~ s{</?font[^>]*?>}{ }gxms;        # bold
+    $text =~ s{</?b[^>]*?>}{ }gxms;           # italic
     $text =~ s{</?i[^>]*?>}{ }gxms;           # italic
     $text =~ s{</?link[^>]*?>}{ }gxms;        # xml link
     $text =~ s{</?p[^>]*?>}{ }gxms;           # html paragraph
@@ -1374,6 +1564,8 @@ sub _clean_block {
 
     # Texinfo end tag (could be more clever but brute force is fast)
     $text =~ s/}/ /gxms;
+    # Tex section titles
+    $text =~ s/^\s*\\(sub)*section\*?\{\s*\S+/ /gxms;
     # single char at end
     # String, C-style comment/javadoc indent,
     # quotes for strings, pipe and backslash, tilde in some txt
@@ -1426,9 +1618,10 @@ sub _md5sum_based_check {
 # check bad license
 sub _license_check {
     my (
-        $source_pkg, $name, $basename,
-        $licensesdatas, $block, $blocknumber,
-        $cleanedblock,$matchedkeyword, $licenseproblemhash
+        $info, $source_pkg, $name,
+        $basename,$licensesdatas, $block,
+        $blocknumber,$cleanedblock,$matchedkeyword,
+        $licenseproblemhash
     )= @_;
     my $ret = 0;
 
@@ -1483,10 +1676,10 @@ sub _license_check {
 
         if(defined($licenseproblemdata->{'callsub'})) {
             my $subresult= $licenseproblemdata->{'callsub'}->(
-                $source_pkg, $name, $basename,
-                $block,$blocknumber,$cleanedblock,
-                $matchedkeyword,$licenseproblemhash,$licenseproblem,
-                %+
+                $info, $source_pkg, $name,
+                $basename,$block,$blocknumber,
+                $cleanedblock,$matchedkeyword,$licenseproblemhash,
+                $licenseproblem,%+
             );
             if($subresult) {
                 $licenseproblemhash->{$licenseproblem} = 1;
@@ -1503,6 +1696,22 @@ sub _license_check {
     return $ret;
 }
 
+sub _ships_examples {
+    my ($group) = @_;
+    my @procs = $group->get_processables('binary');
+    return 1 if not @procs;
+    foreach my $binpkg (@procs) {
+        my $name = $binpkg->pkg_name;
+        # If we have an -examples package, assume we ship examples.
+        return 1 if $name =~ m{-examples$};
+        my @files = $binpkg->info->sorted_index;
+        # Check each package for a directory (or symlink) called "examples".
+        return 1
+          if any { m{^usr/share/doc/(.+/)?examples/?$} } @files;
+    }
+    return;
+}
+
 1;
 
 # Local Variables:
