@@ -27,7 +27,7 @@ use List::MoreUtils qw(any);
 use Text::Levenshtein qw(distance);
 
 use Lintian::Data;
-use Lintian::Relation;
+use Lintian::Relation qw(:constants);
 use Lintian::Tags qw(tag);
 use Lintian::Util qw(strip);
 
@@ -41,19 +41,17 @@ my $maint_commands = Lintian::Data->new('debhelper/maint_commands');
 my $dh_commands_depends = Lintian::Data->new('debhelper/dh_commands', '=');
 my $filename_configs = Lintian::Data->new('debhelper/filename-config-files');
 my $dh_ver_deps= Lintian::Data->new('debhelper/dh_commands-manual', qr/\|\|/o);
-my $dh_addons = Lintian::Data->new('debhelper/dh_addons', '=');
+my $dh_addons = Lintian::Data->new('common/dh_addons', '=');
 my $dh_addons_manual
   = Lintian::Data->new('debhelper/dh_addons-manual', qr/\|\|/o);
 my $compat_level = Lintian::Data->new('debhelper/compat-level',qr/=/);
 
 my $MISC_DEPENDS = Lintian::Relation->new('${misc:Depends}');
-my $NAMED_COMPAT_LEVELS = Lintian::Data->new('debhelper/named-compat-levels',
-    qr/\s*=>\s*/, \&_named_compat_levels);
 
 sub run {
     my (undef, undef, $info) = @_;
     my $droot = $info->index_resolved_path('debian/');
-    my ($drules, $dh_bd_version, $level, $using_named_compat);
+    my ($drules, $dh_bd_version, $level);
 
     my $seencommand = '';
     my $needbuilddepends = '';
@@ -61,17 +59,16 @@ sub run {
     my $needtomodifyscripts = '';
     my $compat = 0;
     my $seendhcleank = '';
-    my %missingbdeps;
-    my %missingbdeps_addons;
-
-    my $maybe_skipping;
-    my $dhcompatvalue;
+    my (%missingbdeps, %missingbdeps_addons, $maybe_skipping, $dhcompatvalue);
     my $inclcdbs = 0;
 
-    my ($bdepends_noarch, $bdepends, %build_systems);
+    my ($bdepends_noarch, $bdepends, %build_systems, $uses_autotools_dev_dh);
+    $bdepends = $info->relation('build-depends-all');
     my $seen_dh = 0;
+    my $seen_dh_parallel = 0;
     my $seen_python_helper = 0;
     my $seen_python3_helper = 0;
+    my %overrides;
 
     $drules = $droot->child('rules') if $droot;
 
@@ -98,6 +95,12 @@ sub run {
             if ($dhcommand eq 'dh_installmanpages') {
                 tag 'dh_installmanpages-is-obsolete', "line $.";
             }
+            if (   $dhcommand eq 'dh_autotools-dev_restoreconfig'
+                or $dhcommand eq 'dh_autotools-dev_updateconfig') {
+                tag 'debhelper-tools-from-autotools-dev-are-deprecated',
+                  "$dhcommand (line $.)";
+                $uses_autotools_dev_dh = 1;
+            }
             if ($dhcommand eq 'dh_python3') {
                 $seen_python3_helper = 1;
             }
@@ -133,14 +136,26 @@ sub run {
             delete($build_systems{'debhelper'});
             $seen_dh = 1;
             $seencommand = 1;
+            $seen_dh_parallel = $. if m/--parallel/;
             $needbuilddepends = 1;
             $needtomodifyscripts = 1;
             while (m/\s--with(?:=|\s+)(['"]?)(\S+)\1/go) {
                 my $addon_list = $2;
                 for my $addon (split(m/,/o, $addon_list)) {
+                    my $orig_addon = $addon;
                     $addon =~ y,-,_,;
-                    my $depends =$dh_addons_manual->value($addon)
+                    my $depends = $dh_addons_manual->value($addon)
                       || $dh_addons->value($addon);
+                    if ($addon eq 'autotools_dev') {
+                        tag
+                          'debhelper-tools-from-autotools-dev-are-deprecated',
+                          "dh ... --with ${orig_addon} (line $.)";
+                        $uses_autotools_dev_dh = 1;
+                    }
+                    tag 'dh-quilt-addon-but-quilt-source-format',
+                      "dh ... --with ${orig_addon}", "(line $.)"
+                      if $addon eq 'quilt'
+                      and $info->field('format', '') eq '3.0 (quilt)';
                     if (defined $depends) {
                         $missingbdeps_addons{$depends} = $addon;
                     }
@@ -182,14 +197,17 @@ sub run {
             my $targets = $1;
             $needbuilddepends = 1;
             # Can be multiple targets per rule.
-            while ($targets =~ /\boverride_(dh_[^\s]+)/g) {
-                my $dhcommand = $1;
+            while ($targets =~ /\boverride_dh_([^\s]+?)(-arch|-indep|)\b/g) {
+                my $cmd = $1;
+                my $arch = $2;
+                my $dhcommand = "dh_$cmd";
+                $overrides{$dhcommand} = [$., $arch];
                 # If maintainer is using wildcards, it's unlikely to be a typo.
                 next if ($dhcommand =~ /%/);
                 next if ($dh_commands_depends->known($dhcommand));
                 # Unknown command, so check for likely misspellings
                 foreach my $x (sort $dh_commands_depends->all) {
-                    if (distance($dhcommand, $x) < 3) {
+                    if ("dh_auto_$cmd" eq $x or distance($dhcommand, $x) < 3) {
                         tag 'typo-in-debhelper-override-target',
                           "override_$dhcommand", '->', "override_$x",
                           "(line $.)";
@@ -209,7 +227,7 @@ sub run {
                   |blends-dev/rules
                  )
               }xsm
-          ) {
+        ) {
             # All of these indirectly use dh.
             $build_systems{'dh'} = 1;
             delete($build_systems{'debhelper'});
@@ -218,7 +236,7 @@ sub run {
               ^include \s+
                  /usr/share/pkg-kde-tools/qt-kde-team/\d+/debian-qt-kde\.mk
               }xsm
-          ) {
+        ) {
             $build_systems{'dhmk'} = 1;
             delete($build_systems{'debhelper'});
         }
@@ -226,7 +244,6 @@ sub run {
     close($rules_fd);
 
     unless ($inclcdbs){
-        my $bdepends = $info->relation('build-depends-all');
         # Okay - d/rules does not include any file in /usr/share/cdbs/
         tag 'unused-build-dependency-on-cdbs' if ($bdepends->implies('cdbs'));
     }
@@ -238,7 +255,10 @@ sub run {
         tag 'debian-build-system', 'other';
     }
 
-    return unless $seencommand;
+    unless ($seencommand) {
+        tag 'package-does-not-use-debhelper-or-cdbs';
+        return;
+    }
 
     my @pkgs = $info->binaries;
     my $single_pkg = '';
@@ -260,22 +280,34 @@ sub run {
     }
 
     my $compatnan = 0;
+    my $compatvirtual;
     my $compat_file = $droot->child('compat');
+    my $visit = sub {
+        return 0 unless m,^debhelper-compat \(= (\d+)\)$,;
+        $level = $1;
+        $compatvirtual = $level;
+        tag 'debhelper-compat-virtual-relation', $compatvirtual;
+        return 1;
+    };
+    $bdepends->visit($visit, VISIT_PRED_FULL | VISIT_STOP_FIRST_MATCH);
+
     # Check the compat file.  Do this separately from looping over all
     # of the other files since we use the compat value when checking
     # for brace expansion.
     if ($compat_file and $compat_file->is_open_ok) {
-        my $compat_file = $compat_file->file_contents;
-        ($compat) = split(/\n/, $compat_file);
-        strip($compat);
+        my $fd = $compat_file->open;
+        while (<$fd>) {
+            if ($. == 1) {
+                $compat = strip($_);
+            } elsif (m/^\d/) {
+                tag 'debhelper-compat-file-contains-multiple-levels',
+                  "(line $.)";
+            }
+        }
+        close($fd);
         if ($compat ne '') {
             my $compat_value = $compat;
-            my $named_compat = $NAMED_COMPAT_LEVELS->value($compat);
-            if (defined($named_compat)) {
-                $dh_bd_version = $named_compat->{'introduced-in'};
-                $compat_value = $named_compat->{'compat-level'};
-                $using_named_compat = 1;
-            } elsif ($compat !~ m/^\d+$/) {
+            if ($compat !~ m/^\d+$/) {
                 tag 'debhelper-compat-not-a-number', $compat;
                 $compat =~ s/[^\d]//g;
                 $compat_value = $compat;
@@ -283,7 +315,6 @@ sub run {
             }
             if ($level) {
                 my $c = $compat;
-                $c .= " ($compat_value)" if $using_named_compat;
                 tag 'declares-possibly-conflicting-debhelper-compat-versions',
                   "rules=$level compat=${c}";
             } else {
@@ -295,7 +326,7 @@ sub run {
             tag 'debhelper-compat-file-is-empty';
         }
     } else {
-        tag 'debhelper-compat-file-is-missing';
+        tag 'debhelper-compat-file-is-missing' unless $compatvirtual;
     }
 
     if (defined($level) and $level !~ m/^\d+$/ and not $compatnan) {
@@ -317,6 +348,17 @@ sub run {
         tag 'dh-clean-k-is-deprecated';
     }
 
+    for my $suffix (qw(enable start)) {
+        my ($line, $arch) = @{$overrides{"dh_systemd_$suffix"} // []};
+        tag 'debian-rules-uses-deprecated-systemd-override',
+          "override_dh_systemd_$suffix$arch", "(line $line)"
+          if $line and $level >= 11;
+    }
+
+    tag 'debian-rules-uses-unnecessary-dh-argument', 'dh ... --parallel',
+      "(line $seen_dh_parallel)"
+      if $seen_dh_parallel and $level >= 10;
+
     # Check the files in the debian directory for various debhelper-related
     # things.
     for my $file ($droot->children) {
@@ -413,8 +455,9 @@ sub run {
                 while (<$fd>) {
                     next if /^\s*$/;
                     next if (/^\#/ and $level >= 5);
-                    if (m/(?<!\\)\{(?:[^\s\\\},]*?,)*[^\\\}\s,]+,*\}/) {
-                        tag 'brace-expansion-in-debhelper-config-file',$file;
+                    if (m/((?<!\\)\{(?:[^\s\\\}]*?,)+[^\\\}\s,]*,*\})/) {
+                        tag 'brace-expansion-in-debhelper-config-file',$file,
+                          $1,"(line $.)";
                         last;
                     }
                 }
@@ -425,8 +468,10 @@ sub run {
 
     $bdepends_noarch = $info->relation_noarch('build-depends-all');
     $bdepends = $info->relation('build-depends-all');
-    if ($needbuilddepends && !$bdepends->implies('debhelper')) {
-        tag 'package-uses-debhelper-but-lacks-build-depends';
+    if ($needbuilddepends) {
+        tag 'package-uses-debhelper-but-lacks-build-depends'
+          unless $bdepends->implies('debhelper')
+          or $bdepends->implies('debhelper-compat');
     }
     if ($needdhexecbuilddepends && !$bdepends->implies('dh-exec')) {
         tag 'package-uses-dh-exec-but-lacks-build-depends';
@@ -443,22 +488,31 @@ sub run {
     while (my ($dep, $addon) = each %missingbdeps_addons) {
         tag 'missing-build-dependency-for-dh-addon', "$addon => $dep"
           unless ($bdepends_noarch->implies($dep));
+
+        # As a special case, the python3 addon needs a dependency on
+        # dh-python unless the -dev packages are used.
+        my $pkg = 'dh-python';
+        tag 'missing-build-dependency-for-dh-addon',"$addon => $pkg"
+          if $addon eq 'python3'
+          && $bdepends_noarch->implies($dep)
+          && !$bdepends_noarch->implies(
+            'python3-dev:any | python3-all-dev:any')
+          && !$bdepends_noarch->implies($pkg);
     }
 
     $dh_bd_version = $level if not defined($dh_bd_version);
-    unless ($bdepends->implies("debhelper (>= ${dh_bd_version}~)")){
+    unless ($bdepends->implies("debhelper (>= ${dh_bd_version}~)")
+        or $bdepends->implies("debhelper-compat (= ${dh_bd_version})")){
         my $tagname = 'package-needs-versioned-debhelper-build-depends';
         my @extra = ($level);
         $tagname = 'package-lacks-versioned-build-depends-on-debhelper'
           if ($dh_bd_version <= $compat_level->value('pedantic'));
-        if ($using_named_compat) {
-            push(@extra, '(for the named compat level)');
-        }
         tag $tagname, @extra;
     }
 
     if ($level >= 10) {
         for my $pkg (qw(dh-autoreconf autotools-dev)) {
+            next if $pkg eq 'autotools-dev' and $uses_autotools_dev_dh;
             tag 'useless-autoreconf-build-depends', $pkg
               if $bdepends->implies($pkg);
         }
@@ -527,7 +581,7 @@ sub _check_dh_exec {
                        ARCH (?: _OS|_CPU|_BITS|_ENDIAN )?
                       |GNU_ (?:CPU|SYSTEM|TYPE)|MULTIARCH
              ) \Z}xsm
-              ) {
+            ) {
                 tag 'dh-exec-subst-unknown-variable', $path, $sv;
             }
         }
@@ -548,7 +602,7 @@ sub _check_dh_exec {
                        ARCH (?: _OS|_CPU|_BITS|_ENDIAN )?
                       |GNU_ (?:CPU|SYSTEM|TYPE)|MULTIARCH
              ) \Z}xsm
-              ) {
+            ) {
                 if (defined($dv)) {
                     $dhe_useless = ($sv eq $dv);
                 } else {
@@ -609,16 +663,6 @@ sub _shebang_cmd {
     return $cmd;
 }
 
-sub _named_compat_levels {
-    my ($key, $raw_val, undef) = @_;
-    my $result = {};
-    for my $opt (split(m/\s*,\s*/, $raw_val)) {
-        my ($key, $val) = split(m/\s*=\s*/, $opt, 2);
-        $result->{$key} = $val;
-    }
-    return $result;
-}
-
 1;
 
 # Local Variables:
