@@ -19,19 +19,19 @@ use warnings;
 use autodie;
 use Carp qw(croak);
 
-use List::MoreUtils qw(any);
+use List::MoreUtils qw(any none);
 
 use Lintian::Data;
 use Lintian::Tags qw(tag);
 use Lintian::Util qw(rstrip);
 
 our $PYTHON_DEPEND
-  = 'python:any | python-dev:any | python-all:any | python-all-dev:any';
+  = 'python:any | python-dev:any | python-all:any | python-all-dev:any | python2:any | python2-dev:any';
 our $PYTHON3_DEPEND
   = 'python3:any | python3-dev:any | python3-all:any | python3-all-dev:any';
 our $PYTHON2X_DEPEND = 'python2.7:any | python2.7-dev:any';
-our $PYTHON3X_DEPEND
-  = join(' | ', map { "python${_}:any | python${_}-dev:any" } qw(3.4 3.5));
+our $PYTHON3X_DEPEND= join(' | ',
+    map { "python${_}:any | python${_}-dev:any" } qw(3.4 3.5 3.6 3.7));
 our $ANYPYTHON_DEPEND
   = "$PYTHON_DEPEND | $PYTHON2X_DEPEND | $PYTHON3_DEPEND | $PYTHON3X_DEPEND";
 
@@ -44,6 +44,10 @@ my $BAD_CONSTRUCT_IN_RULES
   = Lintian::Data->new('rules/rules-should-not-use', qr/\s*~~\s*/,
     sub { return qr/$_[1]/xs });
 
+my $BAD_MULTILINE_CONSTRUCT_IN_RULES
+  = Lintian::Data->new('rules/rules-should-not-use-multiline',
+    qr/\s*~~\s*/,sub { return qr/$_[1]/xsm });
+
 # Certain build tools must be listed in Build-Depends even if there are no
 # arch-specific packages because they're required in order to run the clean
 # rule.  (See Policy 7.6.)  The following is a list of package dependencies;
@@ -57,8 +61,14 @@ our @GLOBAL_CLEAN_DEPENDS = (
     [cdbs => qr'^include\s+/usr/share/R/debian/r-cran\.mk'],
     [dbs => qr'^include\s+/usr/share/dbs/'],
     ['dh-make-php' => qr'^include\s+/usr/share/cdbs/1/class/pear\.mk'],
-    [debhelper => qr'^include\s+/usr/share/cdbs/1/rules/debhelper\.mk'],
-    [debhelper => qr'^include\s+/usr/share/R/debian/r-cran\.mk'],
+    [
+        'debhelper | debhelper-compat' =>
+          qr'^include\s+/usr/share/cdbs/1/rules/debhelper\.mk'
+    ],
+    [
+        'debhelper | debhelper-compat' =>
+          qr'^include\s+/usr/share/R/debian/r-cran\.mk'
+    ],
     [dpatch => qr'^include\s+/usr/share/cdbs/1/rules/dpatch\.mk'],
     ['gnome-pkg-tools' => qr'^include\s+/usr/share/gnome-pkg-tools/'],
     [quilt => qr'^include\s+/usr/share/cdbs/1/rules/patchsys-quilt\.mk'],
@@ -85,7 +95,7 @@ my @GLOBAL_DEPENDS=(
 # depend on, this package is allowed (and required) in Build-Depends.
 my @RULE_CLEAN_DEPENDS =(
     [ant => qr'^\t\s*(\S+=\S+\s+)*ant\s'],
-    [debhelper => qr'^\t\s*dh_.+'],
+    ['debhelper | debhelper-compat' => qr'^\t\s*dh_.+'],
     ['dh-ocaml, ocaml-nox | ocaml' => qr'^\t\s*dh_ocamlinit\s'],
     [dpatch => qr'^\t\s*(\S+=\S+\s+)*dpatch\s'],
     ['po-debconf' => qr'^\t\s*debconf-updatepo\s'],
@@ -158,11 +168,12 @@ sub run {
     # other files, since to chase all includes we'd have to have all
     # of its build dependencies installed.
     local $_;
+    my $build_all = $info->relation('build-depends-all');
     my @arch_rules = (qr/^clean$/, qr/^binary-arch$/, qr/^build-arch$/);
     my @indep_rules = (qr/^build$/, qr/^build-indep$/, qr/^binary-indep$/);
     my (@current_targets, %rules_per_target,  %debhelper_group);
-    my %seen;
-    my $maybe_skipping;
+    my (%seen, %overridden);
+    my ($maybe_skipping, @conditionals);
     my %variables;
     my $uses_makefile_pl = 0;
     my $includes = 0;
@@ -173,6 +184,9 @@ sub run {
         }
         my $line = $_;
 
+        tag 'debian-rules-is-dh_make-template'
+          if $line =~ m/dh_make generated override targets/;
+
         next if /^\s*\#/;
         if (m/^\s*[s-]?include\s+(\S++)/o){
             my $makefile = $1;
@@ -201,7 +215,7 @@ sub run {
         # Check for problems that can occur anywhere in debian/rules.
         if (   m/^\t\s*-(?:\$[\(\{]MAKE[\}\)]|make)\s.*(?:dist)?clean/s
             || m/^\t\s*(?:\$[\(\{]MAKE[\}\)]|make)\s(?:.*\s)?-(\w*)i.*(?:dist)?clean/s
-          ) {
+        ) {
             # Ignore "-C<dir>" (#671537)
             if (not $1 or $1 !~ m,^C,) {
                 tag 'debian-rules-ignores-make-clean-error', "line $.";
@@ -212,11 +226,25 @@ sub run {
             tag 'debian-rules-sets-DEB_BUILD_OPTIONS', "line $.";
         }
 
+        if (
+            /^
+                \s*(?:export\s+)?
+                (DEB_(?:HOST|BUILD|TARGET)_(?:ARCH|MULTIARCH|GNU)[A-Z_]*)\s*:?=
+            /x
+            && keys(%seen) == 0
+        ) {
+            tag 'debian-rules-sets-dpkg-architecture-variable', "$1 (line $.)";
+        }
+
         # check generic problem
         foreach my $bad_construct ($BAD_CONSTRUCT_IN_RULES->all) {
             my $badregex = $BAD_CONSTRUCT_IN_RULES->value($bad_construct);
             if ($line =~ m/$badregex/) {
-                tag $bad_construct, "line $.";
+                if (defined($+{info})) {
+                    tag $bad_construct, $+{info}, "(line $.)";
+                } else {
+                    tag $bad_construct, "line $.";
+                }
             }
         }
 
@@ -230,10 +258,15 @@ sub run {
             # rather well.
             my ($var, $value) = ($1, $2);
             $variables{$var} = $value;
+            tag 'unnecessary-source-date-epoch-assignment', "(line $.)"
+              if $var eq 'SOURCE_DATE_EPOCH'
+              and not $build_all->implies(
+                'dpkg-dev (>= 1.18.8) | debhelper (>= 10.10)');
         }
 
         # Keep track of whether this portion of debian/rules may be optional
-        if (/^ifn?(?:eq|def)\s/) {
+        if (/^ifn?(?:eq|def)\s(.*)/) {
+            push(@conditionals, $1);
             $maybe_skipping++;
         } elsif (/^endif\s/) {
             $maybe_skipping--;
@@ -294,6 +327,7 @@ sub run {
                 qr/^$_$/;
             } split(' ', $target_dependencies);
             for my $target (@current_targets) {
+                $overridden{$1} = $. if $target =~ m/override_(.+)/;
                 if ($target =~ m/%/o) {
                     my $pattern = quotemeta $target;
                     $pattern =~ s/\\%/.*/g;
@@ -381,11 +415,6 @@ sub run {
                     tag 'debian-rules-missing-required-target', $target;
                 } elsif ($typerule eq 'recommended_allindep') {
                     tag 'debian-rules-missing-recommended-target', $target;
-                } elsif ($typerule eq 'goodpractice_dfsg') {
-                    if ($version =~ /(dfsg|debian|ds)/) {
-                        tag 'debian-rules-missing-good-practice-target-dfsg',
-                          $target;
-                    }
                 } else {
                     $typerule ||= '<N/A>';
                     croak(
@@ -411,6 +440,31 @@ sub run {
         tag 'binary-arch-rules-but-pkg-is-arch-indep' if $nonempty;
     }
 
+    foreach my $cmd (qw(dh_clean dh_fixperms)) {
+        foreach my $suffix ('', '-indep', '-arch') {
+            my $line = $overridden{"$cmd$suffix"};
+            tag "override_$cmd-does-not-call-$cmd", "(line $line)"
+              if $line
+              and none { m/^\t\s*-?($cmd\b|\$\(overridden_command\))/ }
+            @{$rules_per_target{"override_$cmd$suffix"}};
+        }
+    }
+
+    if (my $line = $overridden{'dh_auto_test'}) {
+        my @lines = grep {
+                  $_ !~ m{^\t\s*[\:\[]}
+              and $_ !~ m{\bdh_auto_test\b}
+              and $_
+              !~ m{^\t\s*[-@]?(?:(?:/usr)?/bin/)?(?:cp|chmod|echo|ln|mv|mkdir|rm|test|true)}
+        } @{$rules_per_target{'override_dh_auto_test'}};
+        tag 'override_dh_auto_test-does-not-check-DEB_BUILD_OPTIONS',
+          "(line $line)"
+          if @lines and none { m/(DEB_BUILD_OPTIONS|nocheck)/ } @conditionals;
+    }
+
+    tag 'debian-rules-contains-unnecessary-get-orig-source-target'
+      if any { m/^\s+uscan\b/ } @{$rules_per_target{'get-orig-source'}};
+
     # Make sure that all the required build dependencies are there.  Don't
     # issue missing-build-dependency errors for debhelper, since there's
     # another test that does that and it would just be a duplicate.
@@ -444,6 +498,17 @@ sub run {
         }
     }
 
+    $rules_fd = $rules->open;
+    my $sfd = Lintian::SlidingWindow->new($rules_fd);
+    my $block;
+    while ($block = $sfd->readwindow) {
+        foreach my $tag ($BAD_MULTILINE_CONSTRUCT_IN_RULES->all) {
+            my $regex = $BAD_MULTILINE_CONSTRUCT_IN_RULES->value($tag);
+            tag $tag if $block =~ m/$regex/;
+        }
+    }
+    close($rules_fd);
+
     return;
 }
 
