@@ -136,6 +136,7 @@ my @depends_needed = (
     [$update_inetd => '\bupdate-inetd\s'],
     [ucf           => '\bucf\s'],
     ['xml-core'    => '\bupdate-xmlcatalog\s'],
+    ['xfonts-utils' => '\bupdate-fonts-(?:alias|dir|scale)\s'],
 );
 
 my @bashism_single_quote_regexs = (
@@ -247,7 +248,10 @@ sub run {
     my $all_parsed = Lintian::Relation->and($info->relation('all'),
         $info->relation('provides'),$pkg);
     my $str_deps = $info->relation('strong');
-    my $has_sensible_utils = $str_deps->implies('sensible-utils');
+
+    my @x11_fonts
+      = grep {m,^usr/share/fonts/X11/.*\.(?:afm|pcf|pfa|pfb)(?:\.gz)?$,}
+      $info->sorted_index;
 
     for my $filename (sort keys %{$info->scripts}) {
         my $interpreter = $info->scripts->{$filename}{interpreter};
@@ -278,6 +282,12 @@ sub run {
 
         my $is_absolute = ($interpreter =~ m,^/, or defined $calls_env);
 
+        # As a special-exception, Policy 10.4 states that Perl scripts must use
+        # /usr/bin/perl directly and not via /usr/bin/env, etc.
+        tag 'wrong-path-for-interpreter', $filename,
+          '(#!/usr/bin/env perl != /usr/bin/perl)'
+          if defined $calls_env and $interpreter eq 'perl';
+
         # Skip files that have the #! line, but are not executable and
         # do not have an absolute path and are not in a bin/ directory
         # (/usr/bin, /bin etc).  They are probably not scripts after
@@ -346,13 +356,11 @@ sub run {
                 and $filename !~ m,\.erb$,
                 # exclude some shells. zsh -n is broken, see #485885
                 and $base !~ m/^(?:z|t?c)sh$/
-              ) {
+            ) {
 
                 if (check_script_syntax($interpreter, $path)) {
                     script_tag('shell-script-fails-syntax-check', $filename);
                 }
-                check_script_uses_sensible_utils($path)
-                  unless $has_sensible_utils;
             }
         }
 
@@ -417,7 +425,7 @@ sub run {
             $base eq 'perl'
             &&!$str_deps->implies(
                 'libperl4-corelibs-perl | perl (<< 5.12.3-7)')
-          ) {
+        ) {
             my $fd = $path->open;
             while (<$fd>) {
                 if (
@@ -433,7 +441,7 @@ sub run {
                           # ... so they end with ".pl" rather than ".pm"
                           \.pl['"]
                }xsm
-                  ) {
+                ) {
                     tag 'script-uses-perl4-libs-without-dep',
                       "$filename:$. ${1}.pl";
                 }
@@ -453,9 +461,11 @@ sub run {
             }
             if ($depends && !$all_parsed->implies($depends)) {
                 if ($base =~ /^php/) {
-                    tag 'php-script-but-no-php-cli-dep', $filename;
+                    tag 'php-script-but-no-php-cli-dep', $filename,
+                      "#!$interpreter";
                 } elsif ($base =~ /^(python|ruby|[mg]awk)$/) {
-                    tag("$base-script-but-no-$base-dep", $filename);
+                    tag("$base-script-but-no-$base-dep",
+                        $filename, "#!$interpreter");
                 } elsif ($base eq 'csh' && $filename =~ m,^etc/csh/login\.d/,){
                     # Initialization files for csh.
                 } elsif ($base eq 'fish' && $filename =~ m,^etc/fish\.d/,) {
@@ -464,11 +474,14 @@ sub run {
                     $base eq 'ocamlrun'
                     && $all_parsed->matches(
                         qr/^ocaml(?:-base)?(?:-nox)?-\d\.[\d.]+/)
-                  ) {
+                ) {
                     # ABI-versioned virtual packages for ocaml
+                } elsif ($base eq 'escript'
+                    && $all_parsed->matches(qr/^erlang-abi-[\d+\.]+$/)) {
+                    # ABI-versioned virtual packages for erlang
                 } else {
                     tag 'missing-dep-for-interpreter', "$base => $depends",
-                      "($filename)";
+                      "($filename)", "#!$interpreter";
                 }
             }
         } elsif ($VERSIONED_INTERPRETERS->known($base)) {
@@ -482,10 +495,10 @@ sub run {
             my $depends = join(' | ',  @depends);
             unless ($all_parsed->implies($depends)) {
                 if ($base =~ /^(wish|tclsh)/) {
-                    tag "$1-script-but-no-$1-dep", $filename;
+                    tag "$1-script-but-no-$1-dep", $filename, "#!$interpreter";
                 } else {
                     tag 'missing-dep-for-interpreter', "$base => $depends",
-                      "($filename)";
+                      "($filename)", "#!$interpreter";
                 }
             }
         } else {
@@ -494,10 +507,10 @@ sub run {
             $depends =~ s/\$1/$version/g;
             unless ($all_parsed->implies($depends)) {
                 if ($base =~ /^(python|ruby)/) {
-                    tag "$1-script-but-no-$1-dep", $filename;
+                    tag "$1-script-but-no-$1-dep", $filename,"#!$interpreter";
                 } else {
                     tag 'missing-dep-for-interpreter', "$base => $depends",
-                      "($filename)";
+                      "($filename)", "#!$interpreter";
                 }
             }
         }
@@ -626,17 +639,19 @@ sub run {
                 if (check_script_syntax("/bin/${base}", $path)) {
                     tag 'maintainer-shell-script-fails-syntax-check', $file;
                 }
-                check_script_uses_sensible_utils($path)
-                  unless $has_sensible_utils;
             }
         }
 
         # now scan the file contents themselves
         my $fd = $path->open;
 
-        my ($saw_init, $saw_invoke, $saw_debconf,
-            $saw_bange, $saw_sete, $has_code,
-            $saw_statoverride_list, $saw_statoverride_add);
+        my (
+            $saw_init, $saw_invoke,
+            $saw_debconf,$saw_bange,
+            $saw_sete, $has_code,
+            $saw_statoverride_list, $saw_statoverride_add,
+            $saw_udevadm_guard, $saw_update_fonts
+        );
         my %warned;
         my $cat_string = '';
 
@@ -704,10 +719,21 @@ sub run {
                 $seen_helper_cmds{$cmd}{$file} = 1;
             }
 
+            $saw_update_fonts = 1
+              if
+              m,$LEADIN(?:/usr/bin/)?update-fonts-(?:alias|dir|scale)\s(\S+),;
+
+            $saw_udevadm_guard = 1 if m/\b(if|which|command)\s+.*udevadm/g;
+            if (m,$LEADIN(?:/bin/)?udevadm\s, and $saw_sete) {
+                tag 'udevadm-called-without-guard', "$file:$."
+                  unless $saw_udevadm_guard or m/\|\|/;
+            }
+
             if (    m,[^\w](?:(?:/var)?/tmp|\$TMPDIR)/[^)\]}\s],
                 and not m/\bmks?temp\b/
                 and not m/\btempfile\b/
                 and not m/\bmkdir\b/
+                and not m/\bXXXXXX\b/
                 and not m/\$RANDOM/) {
                 #<<< no perltidy - tag name too long
                 tag 'possibly-insecure-handling-of-tmp-files-in-maintainer-script',
@@ -749,7 +775,7 @@ sub run {
                       ($known_shells_regex)\s+-c\s*.+
                     }xsm
                     and $1 ne 'sh'
-                  ) {
+                ) {
                     $within_another_shell = 1;
                 }
                 # if cat_string is set, we are in a HERE document and need not
@@ -769,7 +795,7 @@ sub run {
                                    (?:\"[^\"]+\"|\'[^\']+\')\s*
                                    (?:\&|\||\d?>|<|;|\Z)}xsm
                         and m/^\s*(\.\s+[^\s;\`:]+\s+([^\s;]+))/
-                      ) {
+                    ) {
 
                         my $extra;
                         ($match, $extra) = ($1, $2);
@@ -860,7 +886,7 @@ sub run {
                     # avoid false positives.
                     if ($cat_line
                         =~ m/(?:^|[^<])\<\<\-?\s*(?:[\\]?(\w+)|[\'\"](.*?)[\'\"])/
-                      ) {
+                    ) {
                         $cat_string = $1;
                         $cat_string = $2 if not defined $cat_string;
                     }
@@ -875,6 +901,10 @@ sub run {
                         tag 'read-in-maintainer-script', "$file:$.";
                     }
 
+                    tag 'multi-arch-same-package-calls-pycompile', "$file:$."
+                      if m/^\s*py3?compile(?:\s|\z)/
+                      and $info->field('multi-arch', 'no') eq 'same';
+
                     if (m,>\s*/etc/inetd\.conf(?:\s|\Z),) {
                         tag 'maintainer-script-modifies-inetd-conf',"$file:$."
                           unless $info->relation('provides')
@@ -902,7 +932,7 @@ sub run {
                             $cmd =~ m{ $LEADIN
                                       (/(?:usr/)?s?bin/[\w.+-]+)
                                       (?:\s|;|\Z)}xsm
-                          ) {
+                        ) {
                             tag 'command-with-path-in-maintainer-script',
                               "$file:$. $1";
                         }
@@ -914,7 +944,7 @@ sub run {
                           (?:!\s+)? -x \s+
                           (/(?:usr/)?s?bin/[\w.+-]+)
                           \s+ \]}xsm
-                      ){
+                    ){
                         tag 'command-with-path-in-maintainer-script',
                           "$file:$. $1";
                     }
@@ -955,6 +985,11 @@ sub run {
 
             generic_check_bad_command($_, $file, $., $pkg, 1);
 
+            my $pdepends = $info->relation('pre-depends');
+            tag 'skip-systemd-native-flag-missing-pre-depends', "$file:$."
+              if m/invoke-rc.d\b.*--skip-systemd-native\b/
+              && !$pdepends->implies('init-system-helpers (>= 1.54~)');
+
             if (m,$LEADIN(?:/usr/sbin/)?dpkg-divert\s,
                 && !/--(?:help|list|truename|version)/) {
                 if (/--local/) {
@@ -1015,6 +1050,11 @@ sub run {
             }
         }
 
+        foreach my $font (@x11_fonts) {
+            tag 'missing-call-to-update-fonts', $font
+              if $file eq 'postinst' and not $saw_update_fonts;
+        }
+
         if ($saw_init && !$saw_invoke) {
             tag 'maintainer-script-calls-init-script-directly',
               "$file:$saw_init";
@@ -1230,7 +1270,7 @@ sub script_is_evil_and_wrong {
             # which do not pass on their parameters (i.e. after re-execing
             # they take their parameters (and potentially data) from stdin
             .?(?:\$[{]1:?\+.?)?(?:\$[\@\*])?~x
-          ) {
+        ) {
             $ret = 1;
             last;
         } elsif (/^\s*(\w+)=\$0;/) {
@@ -1245,7 +1285,7 @@ sub script_is_evil_and_wrong {
             .?\$$var.?\s*
             (?:--\s*)?
             .?(?:\$[{]1:?\+.?)?(?:\$[\@\*])?.?\s*\&~x
-          ) {
+        ) {
 
             $backgrounded = 1;
         } elsif (
@@ -1254,7 +1294,7 @@ sub script_is_evil_and_wrong {
             # the exec should either be "eval"ed or a new statement
             (?:^\s*|\beval\s*[\'\"]|(?:;|&&|\b(?:then|else))\s*)
             exec\s+true(?:\s|\Z)~x
-          ) {
+        ) {
 
             $ret = 1;
             last;
@@ -1281,20 +1321,6 @@ sub check_script_syntax {
     return $?;
 }
 
-sub check_script_uses_sensible_utils {
-    my ($path) = @_;
-    my $fd = $path->open;
-    while (<$fd>) {
-        if (m/${LEADIN}(?:select-editor|sensible-(?:browser|editor|pager))\b/){
-            tag 'script-needs-depends-on-sensible-utils',
-              $path->name, "(line $.)";
-            last;
-        }
-    }
-    close($fd);
-    return;
-}
-
 sub remove_comments {
     local $_;
 
